@@ -131,14 +131,28 @@ var gEditItemOverlay = {
                         PlacesUIUtils.getItemDescription(this._paneInfo.itemId));
   },
 
-  _initKeywordField: Task.async(function* (aNewKeyword) {
-    if (!this._paneInfo.isBookmark)
+  _initKeywordField: Task.async(function* (newKeyword = "") {
+    if (!this._paneInfo.isBookmark) {
       throw new Error("_initKeywordField called unexpectedly");
+    }
 
-    let newKeyword = aNewKeyword;
-    if (newKeyword === undefined) {
-      let itemId = this._paneInfo.itemId;
-      newKeyword = PlacesUtils.bookmarks.getKeywordForBookmark(itemId);
+    if (!newKeyword) {
+      let entries = [];
+      yield PlacesUtils.keywords.fetch({ url: this._paneInfo.uri.spec },
+                                       e => entries.push(e));
+      if (entries.length > 0) {
+        // We show an existing keyword if either POST data was not provided, or
+        // if the POST data is the same.
+        let existingKeyword = entries[0].keyword;
+        let postData = this._paneInfo.postData;
+        if (postData) {
+          let sameEntry = entries.find(e => e.postData === postData);
+          existingKeyword = sameEntry ? sameEntry.keyword : "";
+        }
+        if (existingKeyword) {
+          this._keyword = newKeyword = existingKeyword;
+        }
+      }
     }
     this._initTextField(this._keywordField, newKeyword);
   }),
@@ -173,14 +187,25 @@ var gEditItemOverlay = {
   initPanel(aInfo) {
     if (typeof(aInfo) != "object" || aInfo === null)
       throw new Error("aInfo must be an object.");
+    if ("node" in aInfo) {
+      try {
+        aInfo.node.type;
+      } catch (e) {
+        // If the lazy loader for |type| generates an exception, it means that
+        // this bookmark could not be loaded. This sometimes happens when tests
+        // create a bookmark by clicking the bookmark star, then try to cleanup
+        // before the bookmark panel has finished opening. Either way, if we
+        // cannot retrieve the bookmark information, we cannot open the panel.
+        return;
+      }
+    }
 
     // For sanity ensure that the implementer has uninited the panel before
     // trying to init it again, or we could end up leaking due to observers.
     if (this.initialized)
       this.uninitPanel(false);
 
-    let { itemId, itemGuid, isItem,
-          isURI, uri, title,
+    let { itemId, isItem, isURI,
           isBookmark, bulkTagging, uris,
           visibleRows, focusedElement } = this._setPaneInfo(aInfo);
 
@@ -215,7 +240,7 @@ var gEditItemOverlay = {
     }
 
     if (showOrCollapse("keywordRow", isBookmark, "keyword")) {
-      this._initKeywordField();
+      this._initKeywordField().catch(Components.utils.reportError);
       this._keywordField.readOnly = this.readOnly;
     }
 
@@ -366,7 +391,7 @@ var gEditItemOverlay = {
      * set. Then we sort it descendingly based on the time field.
      */
     this._recentFolders = [];
-    for (var i = 0; i < folderIds.length; i++) {
+    for (let i = 0; i < folderIds.length; i++) {
       var lastUsed = annos.getItemAnnotation(folderIds[i], LAST_USED_ANNO);
       this._recentFolders.push({ folderId: folderIds[i], lastUsed: lastUsed });
     }
@@ -380,7 +405,7 @@ var gEditItemOverlay = {
 
     var numberOfItems = Math.min(MAX_FOLDER_ITEM_IN_MENU_LIST,
                                  this._recentFolders.length);
-    for (var i = 0; i < numberOfItems; i++) {
+    for (let i = 0; i < numberOfItems; i++) {
       this._appendFolderItemToMenupopup(menupopup,
                                         this._recentFolders[i].folderId);
     }
@@ -568,7 +593,7 @@ var gEditItemOverlay = {
                     : this._paneInfo.itemGuid;
         PlacesTransactions.EditTitle({ guid, title: newTitle })
                           .transact().catch(Components.utils.reportError);
-      }).catch(Cu.reportError);
+      }).catch(Components.utils.reportError);
     }
   },
 
@@ -601,7 +626,7 @@ var gEditItemOverlay = {
     try {
       newURI = PlacesUIUtils.createFixedURI(this._locationField.value);
     }
-    catch(ex) {
+    catch (ex) {
       // TODO: Bug 1089141 - Provide some feedback about the invalid url.
       return;
     }
@@ -610,7 +635,6 @@ var gEditItemOverlay = {
       return;
 
     if (!PlacesUIUtils.useAsyncTransactions) {
-      let itemId = this._paneInfo.itemId;
       let txn = new PlacesEditBookmarkURITransaction(this._paneInfo.itemId, newURI);
       PlacesUtils.transactionManager.doTransaction(txn);
       return;
@@ -625,14 +649,19 @@ var gEditItemOverlay = {
       return;
 
     let itemId = this._paneInfo.itemId;
-    let newKeyword = this._keywordField.value;
+    let oldKeyword = this._keyword;
+    let keyword = this._keyword = this._keywordField.value;
+    let postData = this._paneInfo.postData;
     if (!PlacesUIUtils.useAsyncTransactions) {
-      let txn = new PlacesEditBookmarkKeywordTransaction(itemId, newKeyword, this._paneInfo.postData);
+      let txn = new PlacesEditBookmarkKeywordTransaction(itemId,
+                                                         keyword,
+                                                         postData,
+                                                         oldKeyword);
       PlacesUtils.transactionManager.doTransaction(txn);
       return;
     }
     let guid = this._paneInfo.itemGuid;
-    PlacesTransactions.EditKeyword({ guid, keyword: newKeyword })
+    PlacesTransactions.EditKeyword({ guid, keyword, postData, oldKeyword })
                       .transact().catch(Components.utils.reportError);
   },
 
@@ -761,6 +790,11 @@ var gEditItemOverlay = {
         this._markFolderAsRecentlyUsed(containerId)
             .catch(Components.utils.reportError);
       }
+
+      // Auto-show the bookmarks toolbar when adding / moving an item there.
+      if (containerId == PlacesUtils.toolbarFolderId) {
+        Services.obs.notifyObservers(null, "autoshow-bookmarks-toolbar", null);
+      }
     }
 
     // Update folder-tree selection
@@ -974,9 +1008,8 @@ var gEditItemOverlay = {
         if (curTagIndex == -1)
           tags.push(tagCheckbox.label);
       }
-      else {
-        if (curTagIndex != -1)
-          tags.splice(curTagIndex, 1);
+      else if (curTagIndex != -1) {
+        tags.splice(curTagIndex, 1);
       }
       this._element("tagsField").value = tags.join(", ");
       this._updateTags();
@@ -1081,7 +1114,7 @@ var gEditItemOverlay = {
       break;
     case "keyword":
       if (this._paneInfo.visibleRows.has("keywordRow"))
-        this._initKeywordField(aValue);
+        this._initKeywordField(aValue).catch(Components.utils.reportError);
       break;
     case PlacesUIUtils.DESCRIPTION_ANNO:
       if (this._paneInfo.visibleRows.has("descriptionRow"))
